@@ -7,19 +7,35 @@
 ### Some low-level helpers.
 ###
 
-fetch_ChromInfo_from_UCSC <- function(genome,
+.fetch_table_from_UCSC <- function(genome, table, colnames=NULL,
         goldenPath_url="http://hgdownload.cse.ucsc.edu/goldenPath")
 {
-    url <- paste(goldenPath_url, genome, "database/chromInfo.txt.gz", sep="/")
+    url <- paste(goldenPath_url, genome,
+                 "database", paste0(table, ".txt.gz"), sep="/")
     destfile <- tempfile()
     download.file(url, destfile, quiet=TRUE)
-    colnames <- c("chrom", "size", "fileName")
-    read.table(destfile, sep="\t", quote="",
-                         col.names=colnames, comment.char="",
-                         stringsAsFactors=FALSE)
+    if (is.null(colnames)) {
+        read.table(destfile, sep="\t", quote="",
+                             comment.char="",
+                             stringsAsFactors=FALSE)
+    } else {
+        read.table(destfile, sep="\t", quote="",
+                             col.names=colnames,
+                             comment.char="",
+                             stringsAsFactors=FALSE)
+    }
 }
 
-### Used in BSgenome!
+### NOT exported but used in the GenomicFeatures package.
+fetch_ChromInfo_from_UCSC <- function(genome,
+    goldenPath_url="http://hgdownload.cse.ucsc.edu/goldenPath")
+{
+    colnames <- c("chrom", "size", "fileName")
+    .fetch_table_from_UCSC(genome, "chromInfo", colnames,
+                           goldenPath_url=goldenPath_url)
+}
+
+### NOT exported but used in the BSgenome package.
 fetch_GenBankAccn2seqlevel_from_NCBI <- function(assembly, AssemblyUnits=NULL)
 {
     assembly_report <- fetch_assembly_report(assembly,
@@ -39,7 +55,7 @@ fetch_GenBankAccn2seqlevel_from_NCBI <- function(assembly, AssemblyUnits=NULL)
 ### fetchExtendedChromInfoFromUCSC()
 ###
 
-.safe_match <- function(query, NCBI_accn)
+..safe_match <- function(query, NCBI_accn)
 {
     hits <- findMatches(query, NCBI_accn)
     q_hits <- queryHits(hits)
@@ -69,7 +85,7 @@ fetch_GenBankAccn2seqlevel_from_NCBI <- function(assembly, AssemblyUnits=NULL)
     query <- sub("-", ".", UCSC_seqlevel, fixed=TRUE)
     query <- paste0(query, accn_suffix)
     names(query) <- UCSC_seqlevel
-    .safe_match(tolower(query), tolower(NCBI_accn))
+    ..safe_match(tolower(query), tolower(NCBI_accn))
 }
 
 .match_UCSC_seqlevel_part2_to_NCBI_accn <- function(UCSC_seqlevel, NCBI_accn,
@@ -89,7 +105,7 @@ fetch_GenBankAccn2seqlevel_from_NCBI <- function(assembly, AssemblyUnits=NULL)
         query[unversioned_idx] <- paste0(query[unversioned_idx], ".1")
     query <- paste0(accn_prefix, query)
     names(query) <- UCSC_seqlevel[idx2]
-    ans[idx2] <- .safe_match(tolower(query), tolower(NCBI_accn))
+    ans[idx2] <- ..safe_match(tolower(query), tolower(NCBI_accn))
     ans
 }
 
@@ -174,6 +190,59 @@ fetch_GenBankAccn2seqlevel_from_NCBI <- function(assembly, AssemblyUnits=NULL)
     ans
 }
 
+.fetch_Ensembl_seqlevels_from_UCSC <-
+    function(genome, Ensembl_mapping_tables, UCSC_seqlevels, duplicated_contigs,
+             goldenPath_url="http://hgdownload.cse.ucsc.edu/goldenPath")
+{
+    stopifnot(identical(Ensembl_mapping_tables, "chromAlias") ||
+              identical(Ensembl_mapping_tables,
+                        c("chromAlias", "ucscToEnsembl")))
+    ensembl_seqlevels <- rep(NA_character_, length(UCSC_seqlevels))
+    names(ensembl_seqlevels) <- UCSC_seqlevels
+
+    ## 1. Fill in what is already provided by the package GenomeInfoDb.
+    org <- suppressWarnings(mapGenomeBuilds(genome)[1L, "organism"])
+    if (!is.null(org)) {
+        ## org2 <- .normalize_organism(org)
+        ## available_orgs <- names(.getNamedFiles())
+        ## if (org2 %in% available_orgs) {
+        chroms <- try(genomeStyles(org), silent=TRUE)
+        if (!is(chroms, "try-error")) {
+            chroms <- chroms[chroms$UCSC %in% names(ensembl_seqlevels), ]
+            ensembl_seqlevels[chroms$UCSC] <- chroms$Ensembl
+        }
+    }
+
+    ## 2. Fetch info from UCSC.
+    for (table in Ensembl_mapping_tables) {
+        uscs_seqlevels_without_ensembl <-
+            names(ensembl_seqlevels)[which(is.na(ensembl_seqlevels))]
+        if (length(uscs_seqlevels_without_ensembl)) {
+            if (table == "chromAlias") {
+                colnames <- c("alias", "chrom", "source")
+            } else {
+                ## 'table' is "ucscToEnsembl"
+                colnames <- c("chrom", "alias")
+            }
+            chroms <- .fetch_table_from_UCSC(genome, table, colnames,
+                                             goldenPath_url=goldenPath_url)
+            if ("source" %in% colnames(chroms)) {
+                idx <- grep("ensembl", chroms[ , "source"])
+                chroms <- chroms[idx, c("chrom", "alias")]
+            }
+            idx <- which(!(chroms[ , "alias"] %in% duplicated_contigs))
+            chroms <- chroms[idx, ]
+            rownames(chroms) <- chroms[ , "chrom"]
+            keep_me <- uscs_seqlevels_without_ensembl %in% rownames(chroms)
+            uscs_seqlevels_without_ensembl <-
+                uscs_seqlevels_without_ensembl[keep_me]
+            ensembl_seqlevels[uscs_seqlevels_without_ensembl] <-
+                chroms[uscs_seqlevels_without_ensembl, "alias"]
+        }
+    }
+    as.character(ensembl_seqlevels)
+}
+
 standard_fetch_extended_ChromInfo_from_UCSC <- function(
                                   genome,
                                   circ_seqs,
@@ -182,11 +251,13 @@ standard_fetch_extended_ChromInfo_from_UCSC <- function(
                                   special_mappings,
                                   unmapped_seqs,
                                   drop_unmapped,
+                                  Ensembl_mapping_tables,
+                                  duplicated_contigs,
                                   goldenPath_url,
                                   quiet)
 {
     chrominfo <- fetch_ChromInfo_from_UCSC(genome,
-                                goldenPath_url=goldenPath_url)
+                                           goldenPath_url=goldenPath_url)
     UCSC_seqlevel <- chrominfo[ , "chrom"]
     circular_idx <- match(circ_seqs, UCSC_seqlevel)
     if (any(is.na(circular_idx)))
@@ -202,6 +273,7 @@ standard_fetch_extended_ChromInfo_from_UCSC <- function(
         if (!quiet)
             warning(wmsg(genome, " UCSC genome is not based on ",
                          "an NCBI assembly"))
+
         oo <- order(rankSeqlevels(ans[ , "UCSC_seqlevel"]))
         ans <- ans[oo, , drop=FALSE]
         rownames(ans) <- NULL
@@ -279,6 +351,18 @@ standard_fetch_extended_ChromInfo_from_UCSC <- function(
         unmapped_idx <- match(unmapped_seqs, ans[ , "UCSC_seqlevel"])
         ans[sort(unmapped_idx), ] <- ans[unmapped_idx, , drop=FALSE]
     }
+    if (!is.null(Ensembl_mapping_tables)) {
+        ## Add mapping to ensembl chromosomes from UCSC tables.
+        Ensembl_seqlevel <- .fetch_Ensembl_seqlevels_from_UCSC(
+                                           genome,
+                                           Ensembl_mapping_tables,
+                                           ans$UCSC_seqlevel,
+                                           duplicated_contigs,
+                                           goldenPath_url=goldenPath_url)
+        if (!all(is.na(Ensembl_seqlevel)))
+            ans <- cbind(ans, Ensembl_seqlevel=Ensembl_seqlevel,
+                              stringsAsFactors=FALSE)
+    }
     rownames(ans) <- NULL
     ans
 }
@@ -293,6 +377,7 @@ SUPPORTED_UCSC_GENOMES <- list(
         circ_seqs="chrM",
         assembly_accession="GCF_000001405.26",
         special_mappings=c(chrM="MT"),
+        Ensembl_mapping_tables=c("chromAlias", "ucscToEnsembl"),
         ## The chromInfo table at UCSC contains sequences that belong to
         ## GRCh38.p12 but not to GRCh38. Because we want to map hg38 to
         ## GRCh38 and not to GRCh38.p12, we drop these sequences.
@@ -313,7 +398,8 @@ SUPPORTED_UCSC_GENOMES <- list(
                            chr6_qbl_hap6="HSCHR6_MHC_QBL_CTG1",
                            chr6_ssto_hap7="HSCHR6_MHC_SSTO_CTG1",
                            chr17_ctg5_hap1="HSCHR17_1_CTG5"),
-        unmapped_seqs=list(`assembled-molecule`="chrM")
+        unmapped_seqs=list(`assembled-molecule`="chrM"),
+        Ensembl_mapping_tables=c("chromAlias", "ucscToEnsembl")
     ),
 
     hg18=list(
@@ -334,7 +420,8 @@ SUPPORTED_UCSC_GENOMES <- list(
         FUN="standard_fetch_extended_ChromInfo_from_UCSC",
         circ_seqs="chrM",
         assembly_accession="GCF_000001515.5",
-        special_mappings=c(chrM="MT")
+        special_mappings=c(chrM="MT"),
+        Ensembl_mapping_tables=c("chromAlias", "ucscToEnsembl")
     ),
 
     panTro3=list(
@@ -360,7 +447,8 @@ SUPPORTED_UCSC_GENOMES <- list(
         FUN="standard_fetch_extended_ChromInfo_from_UCSC",
         circ_seqs="chrM",
         assembly_accession="GCF_000003055.5",
-        special_mappings=c(chrM="MT")
+        special_mappings=c(chrM="MT"),
+        Ensembl_mapping_tables="chromAlias"
     ),
 
     bosTau7=list(
@@ -374,7 +462,8 @@ SUPPORTED_UCSC_GENOMES <- list(
         FUN="standard_fetch_extended_ChromInfo_from_UCSC",
         circ_seqs="chrM",
         assembly_accession="GCF_000003055.4",
-        special_mappings=c(chrM="MT")
+        special_mappings=c(chrM="MT"),
+        Ensembl_mapping_tables=c("chromAlias", "ucscToEnsembl")
     ),
 
 ### Dog
@@ -385,7 +474,8 @@ SUPPORTED_UCSC_GENOMES <- list(
         special_mappings=c(chr1="chr01", chr2="chr02", chr3="chr03",
                            chr4="chr04", chr5="chr05", chr6="chr06",
                            chr7="chr07", chr8="chr08", chr9="chr09",
-                           chrM="MT")
+                           chrM="MT"),
+        Ensembl_mapping_tables="chromAlias"
     ),
 
     canFam2=list(
@@ -401,7 +491,8 @@ SUPPORTED_UCSC_GENOMES <- list(
 ### Ferret
     musFur1=list(
         FUN="standard_fetch_extended_ChromInfo_from_UCSC",
-        assembly_accession="GCF_000215625.1"
+        assembly_accession="GCF_000215625.1",
+        Ensembl_mapping_tables="chromAlias"
     ),
 
 ### Mouse
@@ -410,7 +501,8 @@ SUPPORTED_UCSC_GENOMES <- list(
         circ_seqs="chrM",
         assembly_accession="GCF_000001635.20",
         AssemblyUnits=c("C57BL/6J", "non-nuclear"),
-        special_mappings=c(chrM="MT")
+        special_mappings=c(chrM="MT"),
+        Ensembl_mapping_tables=c("chromAlias", "ucscToEnsembl")
     ),
 
     mm9=list(
@@ -440,7 +532,8 @@ SUPPORTED_UCSC_GENOMES <- list(
         FUN="standard_fetch_extended_ChromInfo_from_UCSC",
         circ_seqs="chrM",
         assembly_accession="GCF_000003025.5",
-        special_mappings=c(chrM="MT")
+        special_mappings=c(chrM="MT"),
+        Ensembl_mapping_tables="chromAlias"
     ),
 
     susScr2=list(
@@ -455,7 +548,11 @@ SUPPORTED_UCSC_GENOMES <- list(
         FUN="standard_fetch_extended_ChromInfo_from_UCSC",
         circ_seqs="chrM",
         assembly_accession="GCF_000001895.5",
-        special_mappings=c(chrM="MT")
+        special_mappings=c(chrM="MT"),
+        Ensembl_mapping_tables="chromAlias"
+        ## https://github.com/ucscGenomeBrowser/kent/blob/master/src/hg/makeDb/doc/rn6.txt
+        ## LINES 856:864
+        , duplicated_contigs=c("AABR07023006.1", "AABR07022518.1")
     ),
 
 ### Rhesus
@@ -479,7 +576,8 @@ SUPPORTED_UCSC_GENOMES <- list(
         assembly_accession="GCF_000002315.3",
         special_mappings=c(chrLGE22C19W28_E50C23="ChrE22C19W28_E50C23",
                            chrLGE64="ChrE64",
-                           chrM="MT")
+                           chrM="MT"),
+        Ensembl_mapping_tables=c("chromAlias", "ucscToEnsembl")
     ),
 
     galGal3=list(
@@ -498,8 +596,9 @@ SUPPORTED_UCSC_GENOMES <- list(
 ### Stickleback
     gasAcu1=list(
         FUN="standard_fetch_extended_ChromInfo_from_UCSC",
-        circ_seqs="chrM"
+        circ_seqs="chrM",
         #assembly_accession="GCA_000180675.1"
+        Ensembl_mapping_tables="chromAlias"
     ),
 
 ### Zebra finch
@@ -522,7 +621,8 @@ SUPPORTED_UCSC_GENOMES <- list(
         FUN="standard_fetch_extended_ChromInfo_from_UCSC",
         circ_seqs="chrM",
         assembly_accession="GCF_000002035.4",
-        special_mappings=c(chrM="MT")
+        special_mappings=c(chrM="MT"),
+        Ensembl_mapping_tables="chromAlias"
     ),
 
     #Too messy!
@@ -546,7 +646,9 @@ SUPPORTED_UCSC_GENOMES <- list(
         FUN="standard_fetch_extended_ChromInfo_from_UCSC",
         circ_seqs="chrM",
         assembly_accession="GCF_000001215.4",
-        special_mappings=c(chrM="MT")
+        ## special_mappings=c(chrM="MT"),
+        special_mappings=c(chrM="mitochondrion_genome"),
+        Ensembl_mapping_tables="chromAlias"
     ),
 
     dm3=list(
@@ -585,7 +687,8 @@ SUPPORTED_UCSC_GENOMES <- list(
         FUN="standard_fetch_extended_ChromInfo_from_UCSC",
         circ_seqs="chrM",
         assembly_accession="GCF_000146045.2",
-        special_mappings=c(chrM="MT")
+        special_mappings=c(chrM="MT"),
+        Ensembl_mapping_tables="chromAlias"
     ),
 
     sacCer2=list(
@@ -599,6 +702,8 @@ fetchExtendedChromInfoFromUCSC <- function(genome,
         goldenPath_url="http://hgdownload.cse.ucsc.edu/goldenPath",
         quiet=FALSE)
 {
+    .Deprecated("getChromInfoFromUCSC")
+
     if (!isSingleString(genome))
         stop("'genome' must be a single string")
     if (!isTRUEorFALSE(quiet))
@@ -615,6 +720,8 @@ fetchExtendedChromInfoFromUCSC <- function(genome,
         special_mappings=supported_genome$special_mappings,
         unmapped_seqs=supported_genome$unmapped_seqs,
         drop_unmapped=supported_genome$drop_unmapped,
+        Ensembl_mapping_tables=supported_genome$Ensembl_mapping_tables,
+        duplicated_contigs=supported_genome$duplicated_contigs,
         goldenPath_url=goldenPath_url,
         quiet=quiet)
 }
